@@ -228,7 +228,7 @@ sub handleVNCStartupAuto {
     # vncStartup was not defined by the user configuration in ~/.vnc/vnc.conf.
     $options->{'vncStartup'} = $vncStartupAuto;
   } elsif (($options->{'src'}{'vncStartup'} // "undef") eq 'cmdline' &&
-	   ($options->{'src'}{'session-orig'} // "undef") ne 'cmdline') {
+           ($options->{'src'}{'session-orig'} // "undef") ne 'cmdline') {
     # Clear config file session arguments if -xstartup specified on cmdline.
     $options->{'src'}{'session-orig'} = 'cmdline';
     $options->{'session-orig'} = $options->{'session'} = [];
@@ -439,19 +439,25 @@ sub getOptionParseTable($$) {
        "if enabled, VNC will only accept connections from localhost." ],
       [63, 'desktop=s'         => 'desktopName',
        "specifies the VNC desktop name." ],
-      [63, 'rfbunixmode=s'     => 'rfbunixmode' ],
-      [63, 'rfbunixpath=s'     => 'rfbunixpath' ],
       [63, 'rfbport=i'         => 'rfbport',
        "provides the TCP port to be used for the RFB protocol." ],
+      [63, 'rfbunixpath=s'     => 'rfbunixpath',
+       "specifies the path of the Unix domain socket to be used for the RFB protocol." ],
+      [63, 'rfbunixmode=s'     => 'rfbunixmode',
+       "specifies the mode of the Unix domain socket, default is 0600." ],
       [63, 'X509Key=s'         => 'X509Key',
        "denotes a X509 certificate key file (PEM format). This is used by the security types X509None, X509Vnc, and X509Plain." ],
       [63, 'X509Cert=s'        => 'X509Cert',
        "denotes the corresponding X509 certificate (PEM format)." ],
+      [63, 'RSAKey=s'          => 'RSAKey',
+       "denotes an RSA key file in PEM format for the RSA-AES security types RA2, RA2ne, RA2_256, and RA2ne_256." ],
+      [63, 'RequireUsername:b' => 'RequireUsername',
+       "denotes if a username is required for the RSA-AES security types, default is off." ],
       [63, 'Password=s'        => 'Password' ],
       [63, 'PasswordFile|rfbauth=s' => 'vncPasswdFile',
        "specifies the password file for security types VncAuth, TLSVnc, and X509Vnc. On default, ~/.vnc/passwd is used." ],
       [63, 'SecurityTypes=s'   => 'SecurityTypes',
-       "specifies a comma list of security types to offer (None, VncAuth, Plain, TLSNone, TLSVnc, TLSPlain, X509None, X509Vnc, X509Plain). On default, offer only VncAuth." ],
+       "specifies a comma list of security types to offer (None, VncAuth, Plain, TLSNone, TLSVnc, TLSPlain, X509None, X509Vnc, X509Plain, RA2, RA2ne, RA2_256, and RA2ne_256). On default, offer only VncAuth." ],
       # Arguments from Xtigervnc (case sensitive)
       [14, 'auth=s'            => 'xauthorityFile' ],
       # -inetd is not handled
@@ -482,6 +488,7 @@ sub getOptionParseTable($$) {
         "specifies the screen area that will be shown to VNC clients. The format is widthxheight+xoffset+yoffset, where `+' signs can be replaced with `-' signs to specify offsets from the right and/or from the bottom of the screen. Offsets are optional, +0+0 is assumed by default (top left corner). If the argument is  empty, full screen is shown to VNC clients (this is the default)." ],
       # Config file only stuff
       [ 2, 'sslAutoGenCertCommand=s' => 'sslAutoGenCertCommand' ],
+      [ 2, 'sslAutoGenRSAKeyCommand=s' => 'sslAutoGenRSAKeyCommand' ],
       [ 2, 'vncUserDir=s'      => 'vncUserDir' ],
       [ 2, 'vncStartup=s'      => 'vncStartup' ],
       [ 2, 'xauthorityFile=s'  => 'xauthorityFile' ],
@@ -1049,18 +1056,17 @@ sub parseCmdLine {
         next;
       } elsif (($arg =~ /^((?:[^-:@=.0-9][\w.-]+@)?(?:\d+\.\d+\.\d+\.\d+|\[[0-9a-fA-F:.]+\]|[^-:@=.0-9][^=:@]*))?(?::(\d+(?:\.\d+)?|\*))?$/) &&
           defined($displayStore) &&
+          (!defined($pendingExtraArg) || ($arg =~ /@/) || ($arg =~ /:(?:\d+(?:\.\d+)?|\*)$/)) &&
           eval { &{$displayStore}('display', $arg); 1 }) {
 #       print STDERR "==> $arg <==\n";
         undef $pendingExtraArg;
         next;
       } elsif ($arg =~ m/^(-([a-zA-Z]))=(.*)$/) {
-        $arg = $1;
         $opt = $2;
         $val = $3;
       } elsif ($arg =~ m/^-([a-zA-Z])$/) {
         $opt = $1;
       } elsif ($arg =~ m/^(--?([a-zA-Z][^=]+))=(.*)$/) {
-        $arg = $1;
         $opt = $par = $2;
         $val = $3;
       } elsif ($arg =~ m/^--?([a-zA-Z][^=]+)$/) {
@@ -1077,7 +1083,7 @@ sub parseCmdLine {
         print STDERR "$PROG: Option $arg: Unrecognized!\n\n";
         last;
       }
-      my $optInfo = $pars{lc($par)} // $opts{$opt//'undef'};
+      my $optInfo = $pars{lc($par//'undef')} // $opts{$opt//'undef'};
       $opt = $par;
       if (defined $optInfo) {
         if ($optInfo->[0] =~ m/=/) {
@@ -1120,7 +1126,7 @@ sub parseCmdLine {
           last;
         }
         undef $pendingExtraArg;
-      } elsif (defined($noPars{lc($par)} // $noOpts{$opt//'undef'})) {
+      } elsif (defined($noPars{lc($par//'undef')} // $noOpts{$opt//'undef'})) {
         $rc = 0;
         print STDERR "$PROG: Option $arg: Unrecognized!\n\n";
         last;
@@ -1324,11 +1330,11 @@ sub usage {
   }
   {
     &lineBreakText($fh, "  ", "\nTo list all active VNC servers of the user use $PROG");
-    &{$usageOptionDumping}(qw(list display rfbport cleanstale));
+    &{$usageOptionDumping}(qw(list display rfbport rfbunixpath cleanstale));
   }
   {
     &lineBreakText($fh, "  ", "\nTo kill a VNC server use $PROG");
-    &{$usageOptionDumping}(qw(kill display rfbport dry-run verbose clean));
+    &{$usageOptionDumping}(qw(kill display rfbport rfbunixpath dry-run verbose clean));
   }
   {
     &lineBreakText($fh, "  ", "\nTo dump version information use $PROG");
@@ -1382,6 +1388,8 @@ sub getConfig {
                # use our own /etc/pam.d/tigervnc as fallback.
       sslAutoGenCertCommand     =>
         "openssl req -newkey ec:$SYSTEMCONFIGDIR/openssl-ecparams.pem -x509 -days 2190 -nodes",
+      sslAutoGenRSAKeyCommand   =>
+        "openssl genrsa -out RSAKeyFile 4096",
 ## Values declared as user values in /etc/tigervnc/vncserver-config-*, i.e.,
 ## values that are intended to be overwritten by ~/.vnc/tigervnc.conf.
       vncUserDir                =>
@@ -1412,6 +1420,10 @@ sub getConfig {
       X509Key                   =>
         undef, # auto generated if absent and stored in
                # ~/.vnc/${HOSTFQDN}-SrvKey.pem
+      RSAKey                    =>
+        undef, # auto generated if absent and stored in
+               # ~/.vnc/${HOSTFQDN}-SrvRsaKey.pem
+      RequireUsername		=> undef,
       session                   => undef,
 # Undocumented values
       vncServerExtraArgs        => [],
